@@ -50,6 +50,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <sys/ioctl.h>
+
+#ifndef EVIOCGRAB
+#define EVIOCGRAB _IOW('E', 0x90, int)
+#endif
 
 #define NEED_EVENTS
 #include <X11/X.h>
@@ -80,6 +85,7 @@
 
 #include "xisb.h"
 
+#include <linux/input.h>
 /*****************************************************************************
  *        Local Headers
  ****************************************************************************/
@@ -129,7 +135,7 @@ static XF86ModuleVersionInfo VersionRec =
         MODINFOSTRING1,
         MODINFOSTRING2,
         XF86_VERSION_CURRENT,
-        0, 8, 6,
+        0, 8, 7,
         ABI_CLASS_XINPUT,
         ABI_XINPUT_VERSION,
         MOD_CLASS_XINPUT,
@@ -285,12 +291,12 @@ void EVTouchProcessAbs(EVTouchPrivatePtr priv)
 
         ev = &priv->ev;
 
-        if (ev->code == ABS_X) {
+        if ( (ev->code == ABS_X) || (ev->code == ABS_Z) ) {
                 priv->cur_x = ev->value;
 		libtouchSetXPos(priv->libtouch, priv->cur_x);
 	}
 
-        if (ev->code == ABS_Y) {
+        if ( (ev->code == ABS_Y) || (ev->code == ABS_RX) ) {
                 priv->cur_y = ev->value;
 		libtouchSetYPos(priv->libtouch, priv->cur_y);
 	}
@@ -504,6 +510,10 @@ DeviceOn (DeviceIntPtr dev)
 
         xf86FlushInput(local->fd);
 
+        if (ioctl(local->fd, EVIOCGRAB, (void *)1))
+                xf86Msg(X_ERROR, "%s: Unable to grab device (%s).\n", local->name, strerror(errno));
+
+
 #ifndef XFREE86_V4
         xf86AddEnabledDevice(local);
 #else
@@ -528,6 +538,7 @@ DeviceOff (DeviceIntPtr dev)
 
         if (local->fd != -1)
         { 
+                ioctl(local->fd, EVIOCGRAB, (void *)0);
                 xf86RemoveEnabledDevice (local);
                 if (priv->buffer)
                 {
@@ -556,10 +567,10 @@ DeviceInit (DeviceIntPtr dev)
         DBGOUT(2, "EVTouch: %s\n", __FUNCTION__);
         LocalDevicePtr local = (LocalDevicePtr) dev->public.devicePrivate;
         EVTouchPrivatePtr priv = (EVTouchPrivatePtr) (local->private);
-        unsigned char map[EV_MAX_BUTTONS + 1];
+        unsigned char map[EV_MAX_BUTTONS];
         int i;
 
-        for (i = 0; i <= EV_MAX_BUTTONS; i++)
+        for (i = 0; i < EV_MAX_BUTTONS; i++)
                 map[i] = i;
 
         priv->btn_count = EV_MAX_BUTTONS;
@@ -599,7 +610,7 @@ DeviceInit (DeviceIntPtr dev)
         if (InitButtonClassDeviceStruct (dev, EV_MAX_BUTTONS, map) == FALSE)
         {
                 ErrorF("Unable to allocate EVTouch touchscreen ButtonClassDeviceStruct\n");
-                return !Success;
+                return BadAlloc;
         } 
 
         DBGOUT(2, "EVTouch: %s btn_count=%d\n", __FUNCTION__, 
@@ -627,14 +638,16 @@ DeviceInit (DeviceIntPtr dev)
                 return !Success;
         }
 
-        InitValuatorAxisStruct (dev, 0, priv->min_x, priv->max_x,
-                                1024,
-                                EV_AXIS_MIN_RES /* min_res */ ,
-                                EV_AXIS_MAX_RES /* max_res */ );
-        InitValuatorAxisStruct (dev, 1, priv->min_y, priv->max_y,
-                                1024,
-                                EV_AXIS_MIN_RES /* min_res */ ,
-                                EV_AXIS_MAX_RES /* max_res */ );
+        xf86InitValuatorAxisStruct(dev, 0, priv->min_x, priv->max_x,
+                                   1024,
+                                   EV_AXIS_MIN_RES /* min_res */ ,
+                                   EV_AXIS_MAX_RES /* max_res */ );
+        xf86InitValuatorDefaults(dev, 0);
+        xf86InitValuatorAxisStruct(dev, 1, priv->min_y, priv->max_y,
+                                   1024,
+                                   EV_AXIS_MIN_RES /* min_res */ ,
+                                   EV_AXIS_MAX_RES /* max_res */ );
+        xf86InitValuatorDefaults(dev, 1);
 
         /* Initial position of pointer on screen: Centered */
         priv->cur_x=(priv->max_x - priv->min_x)/2;
@@ -654,7 +667,6 @@ DeviceInit (DeviceIntPtr dev)
                 return !Success;
         }
 
-
         /* 
          * Allocate the motion events buffer.
          */
@@ -674,8 +686,7 @@ DeviceControl (DeviceIntPtr dev,
         switch (mode)
         {
         case DEVICE_INIT:
-                DeviceInit(dev);
-                RetValue = Success;
+                RetValue = DeviceInit(dev);
                 break;
         case DEVICE_ON:
                 RetValue = DeviceOn(dev);
@@ -911,11 +922,11 @@ ConvertProc ( LocalDevicePtr local,
                 max_y = priv->max_y - priv->min_y;
 
                 if (priv->rotate == EV_ROTATE_NONE) {
-                        screen_width  = priv->phys_width;
-                        screen_height = priv->phys_height;
+                        screen_width  = pScrn->currentMode->HDisplay;
+                        screen_height = pScrn->currentMode->VDisplay;
                 } else {
-                        screen_width  = priv->phys_height;
-                        screen_height = priv->phys_width;
+                        screen_width  = pScrn->currentMode->VDisplay;
+                        screen_height = pScrn->currentMode->HDisplay;
                 }
 
 
@@ -1101,7 +1112,7 @@ EVTouchPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
                 return NULL;
         }
 
-        local->name = dev->identifier;
+        local->name = xstrdup(dev->identifier);
         local->type_name = XI_TOUCHSCREEN;
         local->device_control = DeviceControl;
         local->read_input = ReadInput;
@@ -1124,6 +1135,7 @@ EVTouchPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
 
 	priv->libtouch = xcalloc(1, sizeof(LibTouchRec));
 	libtouchInit(priv->libtouch, local);
+
         priv->screen_num    = xf86SetIntOption(local->options, "ScreenNumber", 0 );
 
         priv->calibrate = xf86SetIntOption(local->options, "Calibrate", 0);
@@ -1218,6 +1230,7 @@ EVTouchPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         xf86ProcessCommonOptions(local, local->options);
         local->flags |= XI86_CONFIGURED;
 
+        xf86CloseSerial(local->fd);
         local->fd = -1;
         return (local);
 }
