@@ -23,11 +23,17 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <misc.h>
 #include <os.h>
 
 #include <xf86.h>
+#ifndef NEED_XF86_TYPES
 #define NEED_XF86_TYPES
+#endif
 #include <xf86_ansic.h>
 #include <xf86_OSproc.h>
 #include <xf86Xinput.h>
@@ -210,8 +216,9 @@ void libtouchInit(LibTouchRecPtr libtouch, LocalDevicePtr local)
 
         /* parse actions for the states */
         for (state_action_idx = 0; state_action_str[state_action_idx] != NULL; state_action_idx++) {
+                DBG(4, ErrorF("LibTouch: Finding Option %s\n",  state_action_str[state_action_idx]));
                 str = xf86FindOptionValue(local->options, state_action_str[state_action_idx]);
-                if (state_action_str == NULL)
+                if (str == NULL)
                         continue;
 
                 for (action_idx = 0; action_str[action_idx] != NULL; action_idx++) {
@@ -222,6 +229,7 @@ void libtouchInit(LibTouchRecPtr libtouch, LocalDevicePtr local)
                 }
         }
 
+
         dump_configuration();
 }
 
@@ -294,6 +302,9 @@ int delta(int x1, int x2)
 static void
 disable_timers(LibTouchRecPtr priv)
 {
+        int sigstate;
+
+        sigstate = xf86BlockSIGIO();
         if (priv->tap_timer) 
                 TimerFree(priv->tap_timer);
         priv->tap_timer = NULL;
@@ -303,6 +314,7 @@ disable_timers(LibTouchRecPtr priv)
                 TimerFree(priv->longtouch_timer);
         priv->longtouch_timer = NULL;
         priv->longtouch_timer_expired = FALSE;
+        xf86UnblockSIGIO(sigstate);
 }
 
 
